@@ -8,11 +8,17 @@ from subprocess import Popen
 import pandas as pd
 from pyunpack import Archive
 from tika import parser
+import py7zr
+import bz2
+import gzip
+import lzma
 
 from texta_parsers.email_parser import EmailParser
 from texta_parsers.settings import META_FIELD
 from texta_parsers.tools.extension import Extension
 from texta_parsers.tools.scanner import DocScanner
+from texta_parsers.tools import rar_extractor
+from texta_parsers.tools import tar_extractor
 from texta_parsers.tools import utils
 from . import exceptions
 
@@ -104,6 +110,35 @@ class DocParser:
         return extracted
 
 
+    def _extract_rar(self, input_path, output_dir):
+        rar_extractor.extract_rar_file(input_path, output_dir)
+
+    def _extract_tar(self, input_path, output_dir):
+        tar_extractor.extract_tar_file(input_path, output_dir)
+
+    def _extract_7z(self, input_path, output_path):
+        with py7zr.SevenZipFile(input_path, mode='r') as z:
+            z.extractall(path=output_path)
+
+    def _extract_bz2(self, input_path, output_path):
+        output_loc = os.path.join(output_path, "out.decompressed")
+        with open(output_loc, 'wb') as out_file, bz2.BZ2File(input_path, 'rb') as in_file:
+            for data in iter(lambda : in_file.read(100 * 1024), b''):
+                out_file.write(data)
+
+    def _extract_gz(self, input_path, output_path):
+        output_loc = os.path.join(output_path, "out.decompressed")
+        with open(output_loc, 'wb') as out_file, gzip.GzipFile(input_path, 'rb') as in_file:
+            for data in iter(lambda : in_file.read(100 * 1024), b''):
+                out_file.write(data)
+
+    def _extract_xz(self, input_path, output_path):
+        output_loc = os.path.join(output_path, "out.decompressed")
+        with open(output_loc, 'wb') as out_file, lzma.LZMAFile(input_path, 'rb') as in_file:
+            for data in iter(lambda : in_file.read(100 * 1024), b''):
+                out_file.write(data)
+
+
     def _extract_archive(self, input_path, output_dir, extension):
         """
         Extracts contents from archives. Works recursively.
@@ -115,7 +150,22 @@ class DocParser:
             os.makedirs(output_dir)
         # extract with digidoc client
         if extension in Extension.DIGIDOC_EXTENSIONS:
-            extracted_paths = self._extract_digidoc(input_path, output_dir)
+            self._extract_digidoc(input_path, output_dir)
+        elif extension == ".rar":
+            self._extract_rar(input_path, output_dir)
+        elif extension == ".tar":
+            self._extract_tar(input_path, output_dir)
+        elif extension == ".7z":
+            self._extract_7z(input_path, output_dir)
+        #Note: as the name of the file in the bz2 archive is unknow, 
+        #extension prediction relies entirely on magic, which tends to fail
+        #in some cases (e.g digidoc)
+        elif extension == ".bz2":
+            self._extract_bz2(input_path, output_dir)
+        elif extension == ".gz":
+            self._extract_gz(input_path, output_dir)
+        elif extension == ".xz":
+            self._extract_xz(input_path, output_dir)
         else:
             archive = Archive(input_path)
             archive.extractall(output_dir)
@@ -186,6 +236,7 @@ class DocParser:
 
             for chunk in reader:
                 for _, row in chunk.iterrows():
+                    row.fillna("", inplace=True)
                     yield row.to_dict()
 
         # .xls or .xlsx
